/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
 *  epix.h -- Automatically created on Fri Oct 22 22:43:02 UTC 2004  *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include <cstdio>
#include <cstdlib>
#include <cmath>

/* 
 * globals.h -- Top-level header file
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 0.8.10
 * Last Change: April 20, 2003
 */

/* 
 * Copyright (C) 2001, 2002, 2003
 * Andrew D. Hwang <ahwang@mathcs.holycross.edu>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#define _EPIX



namespace ePiX {
}

namespace ePiX {

  // Compile-time tweakable constants

  // ePiX's idea of too small/large (10^\pm 4)
  const double EPIX_EPSILON=0.0001;
  const double EPIX_INFTY=10000.0;
  const double EPIX_DBL_INFTY=10000000000.0; // 10^10

  // Output parameters
  const int EPIX_NUM_PTS=80;     // # of points in ellipses, splines
  const int EPIX_FILE_WIDTH=70;  // Width of output file

  // Sizes of markers et. al.
  const double EPIX_SPOT_DIAM=4;
  const double EPIX_DOT_DIAM=3;
  const double EPIX_DDOT_DIAM=2;
  const double EPIX_BOX_SIZE=4;
  const double EPIX_BBOX_SIZE=2;
  const double EPIX_ARROWHEAD_WIDTH=1.5; // Half-width of arrowheads in pt
  const double EPIX_ARROWHEAD_RATIO=5.5; // 2*Length/width ratio of arrowheads

  /*
   * In calculus plotting, determines the size of the increment dt
   * and the number of increments per point printed. A value of 100
   * is adequate for simple plotting, while a value larger than
   * 1000 is not likely to improve the accuracy noticeably.
   *
   * larger -> better accuracy, longer run
   */
  const int EPIX_ITERATIONS=200;

  // Dash length/dot separation parameters
  const double EPIX_DASH_LENGTH=5.0;
  const double EPIX_DOT_SEP=40.0;

  // End of tweakable constans

  // Path styles, data plot markers, and label alignment types
  enum epix_path_style {SOLID, DASHED, DOTTED};

  enum epix_mark_type {PATH, CIRC, SPOT, RING, DOT, DDOT, PLUS, OPLUS, 
		       TIMES, OTIMES, DIAMOND, UP, DOWN, BOX, BBOX};

  enum epix_label_posn {c, r, tr, rt, t, tl, lt, l, bl, lb, b, br, rb};


  // coordinate axis tick marks
  enum epix_tick_type {TICK_NULL, H_AXIS, V_AXIS};

  // "vector field"-type objects
  enum epix_field_type {SLOPE, DART, VECTOR};

  // Deprecated constants

  const int EPIX_PAIRS_PER_LINE=4;/* Number of pairs per line in output file */

  /* 
   * ePiX depicts the rectangle [x_min, x_max] x [y_min, y_max] in a LaTeX
   * figure occupying (h_offset, v_offset) + [0, h_size] x [0, v_size]. If
   * appropriate, orthogonal projection to the plane normal to <viewpoint>
   * is performed.
   */

  /* 
   * <Size> and <offset> of picture in PICTURE coordinates. Sizes 
   * define space in document alotted by LaTeX. Offsets shift the
   * origin from the CENTER of the picture; positive offsets shift
   * RIGHT and UP (contrast LaTeX behavior).
   */
  extern double h_size, v_size, h_offset, v_offset; // zero offset by default

  /*
   * Components of normal vector for 3-d plotting. When set to (0,0,a3),
   * ePiX draws the standard x-y plane in standard orientation. Otherwise
   * ePiX does orthogonal projection to the plane normal to <viewpoint>,
   * keeping the z-axis vertical. Because projection is orthogonal, overall
   * scale in <viewpoint> is immaterial (aside from round-off).
   */
  extern double viewpt1, viewpt2, viewpt3; // default viewpoint (0,0,0)

  // Bounding box of picture in Cartesian coordinates.
  extern double x_min, x_max, y_min, y_max;
  // Cartesian width and height of bounding box
  extern double x_size, y_size;

  // clip_box in Cartesian coordinates
  extern double clip1_min, clip1_max;
  extern double clip2_min, clip2_max;
  extern double clip3_min, clip3_max;

  /*
   * Picture length units
   *
   * <pic_unit> is one of the following valid LaTeX length units: 
   *    cm, in, mm, pc, or pt.
   * <pic_size> is a double that (with pic_unit) specifies unitlength
   */
  extern double pic_size; // e.g., 1
  extern char *pic_unit;  // e.g., "pt" 

  // Shape error types...
  enum constructor_error_type {MALFORMED, MULTIPLICITY, COLLINEAR_PTS};
  enum join_error_type {TANGENT, PARALLEL, COINCIDENT, NON_COPLANAR,
			SEPARATED, CONCENTRIC};

  // and handlers
  struct constructor_error {
    constructor_error_type type;
    constructor_error(constructor_error_type x) { type = x; }
  };

  struct join_error {
    join_error_type type;
    join_error(join_error_type x) { type = x; }
  };

} /* end of namespace */

/*
 * functions.h
 *
 * Andrew D. Hwang            <ahwang@mathcs.holycross.edu>
 *
 * Version 0.8.10rc1
 * Last Change: March 21, 2003
 *
 */



namespace ePiX {

  // ePiX:: trig functions are sensitive to angle units
  double cos(double theta);
  double sin(double theta);
  double tan(double theta); 

  // non-mathematical behavior (i.e. vanishing:) at poles
  double sec(double theta);
  double csc(double theta);
  double cot(double theta);

  // and inverses
  double acos(double arg);
  double asin(double arg);
  double atan(double arg);

  // identity function for angle units
  double angle(double);

  double id (double);

  double proj1 (double, double);
  double proj2 (double, double);

  // utility functions with discontinuities removed
  double sinx (double);
  double sgn (double);
  double recip (double);

  // period-2 extension of absolute value on [-1,1]: \/\/\/
  double cb (double);

  int gcd (int, int);
  double inf (double f(double), double, double);
  double sup (double f(double), double, double);

  double newton (double f(double), double start);

} /* end of namespace */

/* 
 * pairs.h -- ePiX pair:: class and mathematical operators
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 0.8.10rc5
 * Last Change: April 04, 2003
 */

/* 
 * Copyright (C) 2001, 2002, 2003
 * Andrew D. Hwang <ahwang@mathcs.holycross.edu>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */




namespace ePiX {

  class pair 
    {
    public:
      double x1;
      double x2;

      // void constructor
      pair(void) {}

      // Cartesian constructor
      pair(double x1, double x2);

      // copy
      pair(const pair& old);

      // destructor
      ~pair() {}

      // increment operators
      pair& operator += (const pair&);
      pair& operator -= (const pair&);
      pair& operator *= (const double&);
      
      // complex multiplication and division
      pair& operator *= (const pair&);
      pair& operator /= (const pair&);
    }; /* end of pair class */

  int  operator == (const pair&, const pair&);
  pair operator - (const pair&);
  pair operator + (const pair&, const pair&);
  pair operator - (const pair&, const pair&);

  // scalar multiplication
  pair operator * (const double&, const pair&);
  // Multiplication by i
  pair J(pair);

  // complex multiplication and division
  pair operator * (const pair& u, const pair& v);
  pair operator / (const pair& u, const pair& v);

  // dot product
  double operator |(const pair&, const pair&);
  // (a,b)&(x,y)=(ax,by), for unit/coordinate conversion
  pair operator &(const pair&, const pair&);

  struct mesh {
    int n1;
    int n2;
  };

  struct mesh Net(int, int);

  /* * * * Deprecated functions * * * */

  /* Vector operations (now overloaded operators) */
  pair add_pair(pair, pair);

  pair diff_pair(pair, pair);

  pair mult_s(double, pair);

  /* Complex Multiplication */
  pair mult_pair(pair, pair);

} /* end of namespace */

/* 
   triples.h -- ePiX::triple class
 
   This file provides:

   The triple class (ordered triple of <double>s) and operators

    - triple(), P() (default to origin), E_1, E_2, E_3 (standard basis)
    - polar(r, theta), cis(theta), sph(rho, theta, phi)
    - Vector space operators
    - Other vector operators, operands (a,b,c), (x,y,z)
       | dot product                      ax + by + cz
       norm(p) = sqrt(p|p)
       & componentwise product            (ax, by, cz)
       * cross product                    (bz-cy, cx-az, ay-bx)
       J rotate (x1,x2,0)-plane 1/4 turn  (-b,a,0)
       % orthogonalization                u%v = u - ((u|v)/(v|v))*v
 */

/*
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 0.8.10rc7
 * Last Change: April 21, 2003
 */

/* 
 * Copyright (C) 2001, 2002, 2003
 * Andrew D. Hwang <ahwang@mathcs.holycross.edu>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */



namespace ePiX {

  class triple 
    {
    public:
      double x1, x2, x3;

      triple(double arg1=0, double arg2=0, double arg3=0)
	{x1 = arg1; x2 = arg2; x3 = arg3; }

      triple(const triple& old)
      	{ x1 = old.x1; x2 = old.x2; x3 = old.x3; }

     // increment operators
      triple& operator += (const triple& arg)
	{
	  x1 += arg.x1; x2 += arg.x2; x3 += arg.x3;
	  return (*this);
	}

      triple& operator -= (const triple& arg)
	{
	  x1 -= arg.x1; x2 -= arg.x2; x3 -= arg.x3;  
	  return (*this);
	}

      triple& operator *= (const double& c) // scalar multipication
	{
	  x1 *= c; x2 *= c; x3 *= c;
	  return (*this);
	}

      // dot product
      double operator |= (const triple& v)
	{
	  return (this->x1)*(v.x1) + (this->x2)*(v.x2) + (this->x3)*(v.x3);
	}

      // cross product
      triple operator *= (const triple& v)
	{
	  return triple(this->x2 * v.x3 - this->x3 * v.x2,
			this->x3 * v.x1 - this->x1 * v.x3,
			this->x1 * v.x2 - this->x2 * v.x1);
	}
    }; /* end of class */

  inline triple P(double arg1=0, double arg2=0, double arg3=0)
    { return triple(arg1, arg2, arg3); }

  const triple E_1 = P(1,0,0);
  const triple E_2 = P(0,1,0);
  const triple E_3 = P(0,0,1);

  // N.B. ePiX:: trig functions are sensitive to angle units
  inline triple polar(double r, double t) 
    { return P(r*ePiX::cos(t),r*ePiX::sin(t),0); }

  inline triple cis(double t) 
    { return P(ePiX::cos(t), ePiX::sin(t),0); }

  inline triple sph(double r, double t, double phi) 
    {
      return P(r*(ePiX::cos(t))*(ePiX::cos(phi)), 
	       r*(ePiX::sin(t))*(ePiX::cos(phi)), 
	       r*(ePiX::sin(phi)));
    }

  // (in)equality
  inline bool operator == (const triple& u, const triple& v)
  { return ((u.x1 == v.x1) && (u.x2 == v.x2) && (u.x3 == v.x3) ); }

  inline bool operator != (const triple& u, const triple& v)
  { return ((u.x1 != v.x1) || (u.x2 != v.x2) || (u.x3 != v.x3) ); }

  // vector space operations
  inline triple operator - (const triple& u)
  { return P(-u.x1, -u.x2, -u.x3); }
  
  inline triple operator + (const triple& u, const triple& v)
  { return P(u.x1 + v.x1, u.x2 + v.x2, u.x3 + v.x3); }

  inline triple operator - (const triple& u, const triple& v)
  { return P(u.x1 - v.x1, u.x2 - v.x2, u.x3 - v.x3); }

  // scalar multiplication
  inline triple operator * (const double& c, const triple& v)
  { return P(c*(v.x1), c*(v.x2), c*(v.x3)); }

  // cross product
  inline triple operator * (const triple& u, const triple& v)
  {
    return P(u.x2 * v.x3 - u.x3 * v.x2,	
	     u.x3 * v.x1 - u.x1 * v.x3,
	     u.x1 * v.x2 - u.x2 * v.x1);
  }

  inline triple J(const triple arg) { return E_3*arg; }

  // dot product
  inline double operator | (const triple& u, const triple& v)
    { return u.x1*v.x1 + u.x2*v.x2 + u.x3*v.x3; }

  inline double norm(triple arg) { return sqrt(arg|arg); }

  // componentwise product (a,b,c)&(x,y,z)=(ax,by,cz)
  inline triple operator & (const triple& u, const triple& v)
    { return P(u.x1*v.x1, u.x2*v.x2, u.x3*v.x3); }

  // orthogonalization: u%v is component of u perp to v
  inline triple operator % (triple& u, const triple& v)
    { return u - ((u|v)/(v|v))*v; }

} /* end of namespace */

/* 
 * camera.h -- Vectors and operations
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 0.8.10rc7
 * Last Change: April 19, 2003
 */

/* 
 * Copyright (C) 2001, 2002, 2003
 * Andrew D. Hwang <ahwang@mathcs.holycross.edu>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

/*
 * In geometry, a "frame" is a right-handed orthonormal basis, that is,
 * an ordered set of three mututally perpendicular unit vectors, oriented
 * according to the right-hand rule. A frame has nothing to do with
 * picture frames.
 *
 * ePiX uses a camera metaphor to view points in R^3. The camera consists
 * of:
 *   - a frame ({sea, sky, eye}, with the intuitive meanings:)
 *   - a point to look at (the "target")
 *   - the distance from the camera to the target
 *   - a projection mapping from R^3 to R^2 (the "lens")
 *
 * While specific behavior is determined by the lens, the idea is that the
 * viewer sits at given <distance> from the <target>, in the direction of
 * the <eye>. The vectors <sea> and <sky> point horizontally to the right
 * and vertically upward on the screen. A <distance> of zero is taken to
 * mean "infinitely far away"; for example, point projection (<shadow>)
 * becomes orthogonal projection along the <eye> axis. Again, a specific
 * lens can override this behavior, for example by ignoring the distance.
 *
 *
 * This file provides:
 *   - the frame class, and routines for rotating a frame about the axes
 *     determined by its elements. (Note that there are no methods for
 *     rotating about fixed coordinate axes, and that rotations in R^3 do
 *     not generally commute.
 *
 *   - the camera class and routines:
 *       void rotate_sea(double); // change camera orientation
 *       void rotate_sky(double); // about respective axes
 *       void rotate_eye(double);
 *
 *       void range(double d); // set distance from camera to origin
 *       void look_at(P arg); // set target (center of field of view)
 *       void lens(pair (*screen_projection(P))); // set projection mapping
 *
 *       pair shoot(P arg); // project a point to the screen ("take a photo")
 *
 */



namespace ePiX {

  class frame {
  private:
    // orthonormal triple
    triple frame1;
    triple frame2;
    triple frame3;

  public:
    //constructors
    frame(void) { frame1 = E_1; frame2 = E_2; frame3 = E_3; }

    // Gram-Schmidt
    frame(triple arg1, triple arg2, triple arg3);

    // frame elements
    triple sea() { return frame1; }
    triple sky() { return frame2; }
    triple eye() { return frame3; }

    // rotations about frame elements
    frame rotate_frame1(double angle)
      {
	triple temp2 = frame2;
	triple temp3 = frame3;

	frame2 = 
	  (ePiX::cos(angle)*(temp2)) - (ePiX::sin(angle)*(temp3));
	frame3 = 
	  (ePiX::sin(angle)*(temp2)) + (ePiX::cos(angle)*(temp3));

	return *this;
      }


    frame rotate_frame2(double angle)
      {
	triple temp3 = frame3;
	triple temp1 = frame1;

	frame3 = 
	  (ePiX::cos(angle)*(temp3)) - (ePiX::sin(angle)*(temp1));

	frame1 = 
	  (ePiX::sin(angle)*(temp3)) + (ePiX::cos(angle)*(temp1));

	return *this;
      }

    frame rotate_frame3(double angle)
      {
	triple temp1 = frame1;
	triple temp2 = frame2;

	frame1 = 
	  (ePiX::cos(angle)*(temp1)) - (ePiX::sin(angle)*(temp2));

	frame2 = 
	  (ePiX::sin(angle)*(temp1)) + (ePiX::cos(angle)*(temp2));

	return *this;
      }

  }; // end of class frame

  class epix_camera {
  private:
    frame orientation;
    double distance;
    triple target;
    pair (*screen_projection)(triple);

  public:

    // defaults to orthog projection on (x,y) plane from pos z-axis
    epix_camera(void);

    double get_range(void) { return distance; }
    triple get_target(void) { return target; }

    // pitch: rotate camera up/down
    void rotate_sea(double angle)
      { 
	orientation = orientation.rotate_frame1(angle);
      }

    // yaw: rotate camera left/right
    void rotate_sky(double angle)
      { 
	orientation = orientation.rotate_frame2(angle);
      }

    // roll: rotate camera about viewing axis
    void rotate_eye(double angle)
      { 
	orientation = orientation.rotate_frame3(angle);
      }

    // get/set distance from camera to origin
    void range(double d) { distance = d; }

    // set target (center of field of view)
    void look_at(triple arg) { target = arg; }

    // set projection mapping
    void lens(pair proj(triple)) { screen_projection = proj; }

    // project a point to the screen ("shoot a photo")
    pair shoot(triple arg) { return screen_projection(arg); }

    // get/set eye vector
    friend triple get_sea(epix_camera camera);
    friend triple get_sky(epix_camera camera);
    friend triple get_eye(epix_camera camera);

    friend void viewpoint(double a1, double a2, double a3);
    
  }; // end of class epix_camera 

  void viewpoint(triple arg);

  // screen projection mappings ("lenses")
  pair shadow(triple arg);
  pair fisheye(triple arg);
  pair bubble(triple arg);

  extern epix_camera camera;
  // Project <triple> according to current camera
  inline pair view(triple arg) { return camera.shoot(arg); }

} /* end of namespace */

/* 
   segment.h -- ePiX::segment class
 
   This file provides:

   The segment class (unordered pair of points),
   and operators:
    - join() (alternative constructor)
    - endpt1(), endpt2() (no guaranteed order)
    - segment += triple (translate by <triple>)
    - draw() (ePiX line)
    - midpoint(); (also callable on pair of triples)
    - segment*segment (point of intersection of _lines_ determined by segments)
 */

/*
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 0.8.10rc7
 * Last Change: April 21, 2003
 */

/* 
 * Copyright (C) 2001, 2002, 2003
 * Andrew D. Hwang <ahwang@mathcs.holycross.edu>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */



namespace ePiX {

  class segment
    {
    private:
      triple endpt1;
      triple endpt2;

    public:
      segment() {};
      segment(triple p1, triple p2) { endpt1 = p1; endpt2 = p2; }

      // Note: Ends of segment are not "ordered" meaningfully
      triple end1() const { return endpt1; }
      triple end2() const { return endpt2; }

      // translation
      segment& operator += (const triple& arg)
	{
	  endpt1 += arg;
	  endpt2 += arg;

	  return *this;
	}

      // segment
      void draw();

    }; /* end of segment class */

  inline triple midpoint(segment arg) 
    { 
      return 0.5*(arg.end1() + arg.end2()); 
    }

  // translation
  inline segment& operator + (const segment& seg, const triple& arg)
    {
      segment temp=seg;
      return temp += arg;
    }

  inline segment& operator + (const triple& arg, const segment& seg)
    {
      segment temp=seg;
      return temp += arg;
    }

  // intersection
  triple& operator * (const segment& seg1, const segment& seg2);


  // alternative constructor
  inline segment join(triple p1, triple p2) { return segment(p1,p2); }

  // midpoint of pair of triples      
  inline triple midpoint(triple arg1, triple arg2) 
    { 
      return 0.5*(arg1 + arg2); 
    }

} /* end of namespace */

/* 
   circle.h -- ePiX::circle class
 
   This file provides:

   The circle class (center, radius, unit normal) and operators. See
   function declarations for list of exceptions thrown

    - circle() (defaults to origin, unit radius, in (x1,x2,0)-plane
    - circle(ctr, pt) (parallel to (x1,x2,0)-plane, center ctr, through pt)
    - circle(pt1, pt2, pt3) (arbitrary non-collinear points)
    - center(), radius(), perp()
    - circle += triple (translate by <triple>)
    - circle *= double (scale by <double>)
    - draw() (ePiX ellipse)
    - circle*segment, circle*circle (segment def'd by pts of intersection)
 */

/* 
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 0.8.10rc7
 * Last Change: April 21, 2003
 */

/* 
 * Copyright (C) 2001, 2002, 2003
 * Andrew D. Hwang <ahwang@mathcs.holycross.edu>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */



namespace ePiX {

  class circle 
    {
    private:
      triple ctr;
      double rad;
      triple perp_to; // *unit* normal, even if rad = 0

    public:
      // defaults to unit circle in (x1,x2,0) plane
      circle(triple arg1=P(0,0,0), double arg2=1, triple arg3=E_3) 
	{ 
	  ctr = arg1;
	  rad = arg2; 
	  perp_to = arg3;
	}
      circle(triple ctr, triple pt); // center, and point, normal = E_3
      circle(triple pt1, triple pt2, triple pt3); // three points

      triple center() const { return ctr; }
      double radius() const { return rad; }
      triple perp()   const { return perp_to; }

      // translation
      circle& operator += (const triple& arg)
	{
	  ctr += arg;

	  return *this;
	}

      // scaling
      circle& operator *= (const double& arg)
	{
	  rad *= arg;

	  return *this;
	}

      // circle
      void draw();

    }; /* end of circle class */

  // intersection operators
  /*
   * Throws the following exceptions:
   *   NON_COPLANAR if segment, circle are not coplanar
   *   SEPARATED if coplanar but line defined by segment misses circle
   *   TANGENT if coplanar but line defined by segment tangent to circle
   */
  segment& operator * (const segment&, const circle&);

  /*
   * Throws the following exceptions:
   *   TANGENT if circles are tangent
   *   COINCIDENT if circles are identical
   *   NOT_COPLANAR if circles not coplanar
   *   SEPARATED if disjoint, neither contained in the other
   *   CONCENTRIC if disjoint, one contained in other (not nec same ctr)
   */
  segment& operator * (const circle&, const circle&);

  inline segment operator * (const circle& arg_circle, const segment& arg_seg)
    { return arg_seg*arg_circle; }

  // affine operations: translation by a triple...
  inline circle& operator + (const triple& displace, const circle& circ)
    { 
      circle temp = circ;
      return temp += displace; 
    }
  inline circle& operator + (const circle& circ, const triple& displace)
    { 
      circle temp = circ;
      return temp += displace; 
    }

  // and scaling by a double
  inline circle& operator * (const double& c, const circle& circ)
    { 
      circle temp = circ;
      return temp *= c; 
    }
  inline circle& operator * (const circle& circ, const double& c)
    { 
      circle temp = circ;
      return temp *= c; 
    }

} /* end of namespace */

/*
 * lengths.h -- ePiX true and LaTeX length manipulation
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 0.8.10rc5
 * Last Change: April 06, 2003
 */

/* 
 * Copyright (C) 2001, 2002, 2003
 * Andrew D. Hwang <ahwang@mathcs.holycross.edu>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
 * In ePiX, there are three systems of coordinates:              *
 *                                                               *
 *   Cartesian (C) -- All the user needs to know about           *
 *   Picture   (P) -- The language of the LaTeX front end        *
 *   True      (T) -- Physical location on page from lower left  *
 *                                                               *
 * The coordinate triumvirates' initials form a bad physics pun. *
 *                                                               *
 * Recognized LaTeX units are: cm, in, mm, pc, and pt            *
 *                                                               *
 * All objects and locations are specified in C by the user; the *
 * conversion to true coordinates is transparent. The rationale  *
 * for retaining LaTeX coordinates, rather than forcing all ePiX *
 * figures to be specified in true pt, is of course that figures *
 * are sized according to printing requirements, and often not   *
 * in terms of true pt. The reason for using true pt is that raw *
 * scaling of a LaTeX figure is not robust; label positions must *
 * be specified with true pt offsets if their locations are to   *
 * scale properly when a figure is resized.                      *
 *                                                               *
 * An ePiX figure is a rectangle, described as follows:          *
 *   Cartesian: [x_min, x_max] x [y_min, y_max]                  *
 *   Picture:   [0, h_size] x [0, v_size] + (h_offset, v_offset) *
 *   True:      Only (La)TeX knows/needs to know this            *
 *                                                               *
 * ePiX provides six scale-changing functions:                   *
 *   c2p, p2c -- Cartesian -> LaTeX and back for points          *
 *   c2s, s2c -- Cartesian -> LaTeX and back for vectors         *
 *   p2t, t2p -- LaTeX -> True and back                          *
 *                                                               *
\* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */




namespace ePiX {

  // If less than EPIX_EPSILON, set to zero 
  double truncate(double);

  pair truncate(pair);

  // Coordinate conversion routines
  pair c2p(pair);
  pair p2c(pair);

  pair c2s(pair);
  pair s2c(pair);

  double p2t(double);
  double t2p(double);
  pair p2t(pair);
  pair t2p(pair);

  double raw_len(pair); // Pythagorean length of pair/triple
  double raw_len(triple);

  // functions of <pair> in picture coords
  double true_length(pair); // length in true pt
  pair true_unit(pair);     // parallel true pt unit vector

} /* end of namespace */

/*
 * output.h -- ePiX file-formatting output routines
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 0.8.10rc9
 * Last Change: April 24, 2003
 */

/* 
 * Copyright (C) 2001, 2002, 2003
 * Andrew D. Hwang <ahwang@mathcs.holycross.edu>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */



namespace ePiX {

  extern epix_path_style EPIX_PATH_STYLE;
  extern double epix_stretch_factor;

  // Functions for declaring variables
  void bounding_box(triple, triple);
  void clip_box(triple, triple);
  void clip_to(triple);  // one corner at origin
  void clip_box(triple); // symmetric corners
  void picture(triple);
  void unitlength (const char *);
  void offset(triple);

  // Cropping test (inside bounding_box)
  bool is_visible(triple arg);
  // Clipping test (inside clip_box)
  bool is_in_bounds(triple arg);

  // Sectioning Functions

  // Begin/end picture
  void begin(void);
  void end(void);
  
  // Print Cartesian pair in form (p.x1,p.x2)
  void print(triple);
  void print(pair);
  void print(triple location, triple offset);
  void print_vector(triple);

  void stretch(double);

  // set angle units
  void radians(void);
  void degrees(void);
  void revolutions(void);

  // Path style declarations
  void start_path(void);
  // (Un)fill closed path

  inline void fill(void)   { fprintf (stdout, "\\blacken"); }
  inline void unfill(void) { fprintf (stdout, "\\whiten"); }

  void epix_comment(char *);
  void end_stanza(void);
  void line_break(int, int);

  // Misc Styles and output formatting
  // Output styles
  void pen(char *);
  void pen(double);

  inline void bold(void)     { fprintf (stdout, "\n\\thicklines"); }
  inline void endbold(void)  { fprintf (stdout, "\n\\thinlines"); }
  inline void end_bold(void) { fprintf (stdout, "\n\\thinlines"); }
  inline void plain(void)    { fprintf (stdout, "\n\\thinlines"); }

  // set state variable
  inline void solid(void)  { EPIX_PATH_STYLE = SOLID; }
  inline void dashed(void) { EPIX_PATH_STYLE = DASHED; }
  inline void dotted(void) { EPIX_PATH_STYLE = DOTTED; }

  inline void dashed(double t) 
    { 
      EPIX_PATH_STYLE = DASHED; 
      epix_stretch_factor = t;
    }
  inline void dotted(double t) 
    { 
      EPIX_PATH_STYLE = DOTTED; 
      epix_stretch_factor = t;
    }

  /*
   * Color handling
   *
   * ePiX provides  "pstcol" style colors, which are declarations rather 
   * than delimited environments. To make part of an ePiX file print red 
   * (say), use one of the following:
   *
   *   red;
   *   <epix commands>
   *   end_red;
   *
   *   rgb(1,0,0);
   *   <epix commands>
   *   black;
   *
   * rgb and cmyk colors are specified (respectively) by 3 or 4 floats,
   * the densities of the corresponding primary colors. If these densities
   * are specified outside the unit interval [0,1], they will be truncated,
   * e.g., rgb(2, -1, 0.3) = rgb(1, 0, 0.3).
   */

  // red-green-blue
  void rgb(double r, double g, double b); 
  // cyan-magenta-yellow-black
  void cmyk(double c, double m, double y, double k);

  // primary colors

  inline void red(void)     { fprintf (stdout, "\n\\color[rgb]{1,0,0}"); }
  inline void green(void)   { fprintf (stdout, "\n\\color[rgb]{0,1,0}"); }
  inline void blue(void)    { fprintf (stdout, "\n\\color[rgb]{0,0,1}"); }
  inline void white(void)   { fprintf (stdout, "\n\\color[rgb]{1,1,1}"); }

  inline void cyan(void)    { fprintf (stdout, "\n\\color[cmyk]{1,0,0,0}"); }
  inline void magenta(void) { fprintf (stdout, "\n\\color[cmyk]{0,1,0,0}"); }
  inline void yellow(void)  { fprintf (stdout, "\n\\color[cmyk]{0,0,1,0}"); }
  inline void black(void)   { fprintf (stdout, "\n\\color[cmyk]{0,0,0,1}"); }

  /* 
   * depth of gray in shaded objects; argument is an integer from 0
   * (white) to 32 (black) 
   */
  void gray(double);

} /* end of namespace */

/* 
 * objects.h -- Simple picture objects: axes, grids, markers, polygons
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 0.8.10rc10
 * Last Change: June 04, 2003
 */

/* 
 * Copyright (C) 2001, 2002, 2003
 * Andrew D. Hwang <ahwang@mathcs.holycross.edu>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */




namespace ePiX {

  // Picture objects

  /*
   * label -- put string constant <label_text> at Cartesian position <base>
   * translated by (offset.x1, offset.x2) true points (i.e., 3rd component
   * of <offset> is discarded).
   *
   * Accepts an optional LaTeX-style positioning argument.
   * If no offset is specified, the label is centered at the given Cartesian
   * location.
   * masklabel requires the "color" package, and places the text in 
   * a white box that masks whatever is underneath and earlier in the file.
   */
  // location given as a <triple>
  void label(triple base, triple offset, char *label_text);
  void label(triple base, char *label_text);
  void label(triple base, triple offset, char *label_text,
	     epix_label_posn);

  void masklabel(triple base, triple offset, char *label_text);
  void masklabel(triple base, char *label_text);
  void masklabel(triple base, triple offset, char *label_text,
		 epix_label_posn);

  // Empty and filled LaTeX circles of diameter EPIX_SPOT_DIAM true pt
  void circ(triple); // filled white circ 4 pt in diameter
  void ring(triple, double); // unfilled circ of specified diam
  void spot(triple);
  void dot(triple);
  void ddot(triple);
  void dot(triple, double); // filled circ of specified diam
  void marker (triple, epix_mark_type);

  // Axes and coordinate grids.

  /* 
   * Coordinate axes, specified by initial and final points, number of
   * tick marks, and (optionally) number of points (for cameras that do
   * not map lines to lines). Generally num_pts should be a multiple of n.
   * h/v_axis are identical except for style of tick marks.
   */
  void h_axis_tick(triple location);
  void v_axis_tick(triple location);

  //  void h_axis(triple tail, triple head, int n);
  //  void v_axis(triple tail, triple head, int n);

  // n+1 = #ticks, num_pts = #segments used to draw
  void h_axis(triple tail, triple head, int n, int num_pts=1);
  void v_axis(triple tail, triple head, int n, int num_pts=1);

  // These versions may be used with gcc 2.96 and later
  /*
  inline void h_axis(int n=(int)x_size) { h_axis(P(x_min,0), P(x_max,0), n); }
  inline void v_axis(int n=(int)y_size) { v_axis(P(0,y_min), P(0,y_max), n); }
  */
  inline void h_axis(int n=x_size) { h_axis(P(x_min,0), P(x_max,0), n); }
  inline void v_axis(int n=y_size) { v_axis(P(0,y_min), P(0,y_max), n); }

  // n+1 = #ticks, num_pts = #segments used to draw
  inline void h_axis(int n, int num_pts) 
    { h_axis(P(x_min,0), P(x_max,0), n, num_pts); }
  inline void v_axis(int n, int num_pts) 
    { v_axis(P(0,y_min), P(0,y_max), n, num_pts); }

  /*
   * h_axis_labels: Draws n+1 equally-spaced axis labels between 
   *    <tail> and <head>. Automatically generates label values from
   *    Cartesian coordinates, and offsets labels in true pt.
   */

  void h_axis_labels(triple tail, triple head, int n, triple offset);
  void v_axis_labels(triple tail, triple head, int n, triple offset);

  void h_axis_masklabels(triple tail, triple head, int n, triple offset);
  void v_axis_masklabels(triple tail, triple head, int n, triple offset);

  // Axis labels with LaTeX-style alignment option

  void h_axis_labels(triple tail, triple head, int n, 
		     triple offset, epix_label_posn POSN);
  void v_axis_labels(triple tail, triple head, int n, 
		     triple offset, epix_label_posn POSN);

  void h_axis_masklabels(triple tail, triple head, int n, 
			 triple offset, epix_label_posn POSN);
  void v_axis_masklabels(triple tail, triple head, int n, 
			 triple offset, epix_label_posn POSN);

  // Axis labels with default endpoints
  inline void h_axis_labels(int n, triple offset)
    { h_axis_labels(P(x_min,0), P(x_max,0), n, offset); }
  inline void h_axis_masklabels(int n, triple offset)
    { h_axis_masklabels(P(x_min,0), P(x_max,0), n, offset); }

  inline void h_axis_labels(int n, triple offset, epix_label_posn POSN)
    { h_axis_labels(P(x_min,0), P(x_max,0), n, offset, POSN); }
  inline void h_axis_masklabels(int n, triple offset, epix_label_posn POSN)
    { h_axis_masklabels(P(x_min,0), P(x_max,0), n, offset, POSN); }


  inline void v_axis_labels(int n, triple offset)
    { v_axis_labels(P(0,y_min), P(0,y_max), n, offset); }
  inline void v_axis_masklabels(int n, triple offset)
    { v_axis_masklabels(P(0,y_min), P(0,y_max), n, offset); }

  inline void v_axis_labels(int n, triple offset, epix_label_posn POSN)
    { v_axis_labels(P(0,y_min), P(0,y_max), n, offset, POSN); }
  inline void v_axis_masklabels(int n, triple offset, epix_label_posn POSN)
    { v_axis_masklabels(P(0,y_min), P(0,y_max), n, offset, POSN); }



  // eepic.sty's ellipse comand
  void draw_native_ellipse (triple, triple);
  void native_ellipse (triple, triple);

  /*
   * A "mesh" is an ordered pair of positive integers, and is used to
   * specify the "fineness" of a mesh. Grids, like parametric surface
   * meshes, have a "coarse mesh" -- the numbers of grid intervals in
   * each direction, and a "fine mesh" -- the numbers of points used
   * to render the grid lines. Since an ePiX camera does not always
   * map lines in object space to lines on the screen, grid lines cannot
   * generally be drawn using only two points.
   * A grid may look strange unless each component of fine is a multiple
   * of the corresponding entry of coarse, of course. :)
   */

  // Cartesian grid of specified size, mesh, and resolution
  void draw_grid(triple arg1, triple arg2, mesh coarse, mesh fine);
  void grid(triple arg1, triple arg2, mesh coarse, mesh fine);

  // coarse = fine = (n1,n2)
  void grid(triple arg1, triple arg2, int n1, int n2);
  // arg1 = P(x_min, y_min), arg2 = P(x_max, y_max)
  void grid(int n1, int n2);

  // polar grid of specified radius, mesh (rings and sectors), and resolution
  void draw_polar_grid(double r, mesh coarse, mesh fine);
  void polar_grid(double r, mesh coarse, mesh fine);

  // polar grid with n1 rings and n2 sectors
  void polar_grid(double r, int n1, int n2);

  // Simple geometric objects

  // lines can be "stretched" by double parameter
  void draw_line(triple, triple, int, double); 

  // lines with stretch factor
  void line(triple, triple, double);     

  // Unstretched lines
  void line(triple, triple);    

  inline void draw(segment arg) { line(arg.end1(), arg.end2()); }

  // Returns >0 iff p0 is left of directed segment from p1 to p2
  double ccw(triple p0, triple p1, triple p2);
  // "Visible" portion of the line through p1, p2
  void draw_Line(triple, triple, int);
  void Line(triple, triple);

  // point-slope form
  void Line(triple, double);

  // triangles and quadrilaterals
  inline void draw_triangle(triple arg1, triple arg2, triple arg3)
  {
    start_path();
    print(arg1);
    print(arg2);
    print(arg3);
    print(arg1);
  }
  void triangle(triple, triple, triple);

  inline void draw_quad(triple arg1, triple arg2, triple arg3, triple arg4)
  {
    start_path();
    print(arg1);
    print(arg2);
    print(arg3);
    print(arg4);
    print(arg1);
  }
  void quad(triple, triple, triple, triple);

  void draw_rect(triple, triple); // opposite corners
  void rect(triple, triple);
  // filled rectangle
  void swatch(triple, triple);

  // polygon/polyline with variable number of vertices
  void draw_polygon(int num_pts ...); // data points only
  void polygon(int num_pts ...);
  void polyline(int num_pts ...);

  void draw_arrowhead(triple tail, triple head, double width, double scale=1);
  void draw_shaft(triple tail, triple head, double width, double scale=1);
  void draw_arrow(triple tail, triple head, double scale=1);
  void draw_boldarrow(triple tail, triple head, double scale=1);
  void arrow(triple tail, triple head, double scale=1);
  void boldarrow(triple tail, triple head, double scale=1);

  void draw_dart(triple, triple);
  void draw_bolddart(triple, triple);
  void dart(triple, triple);
  void bolddart(triple, triple);

} /* end of namespace */

/*
 * curves.h -- Ellipses, arcs, splines
 *
 * Andrew D. Hwang            <ahwang@mathcs.holycross.edu>
 *
 * Version 0.8.10rc11
 * Last Change: June 04, 2003
 *
 */

/* 
 * Copyright (C) 2001, 2002, 2003
 * Andrew D. Hwang <ahwang@mathcs.holycross.edu>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */



namespace ePiX {

  // Ellipses, half-ellipses, circular arcs, splines -- curves.c

  /* * * Deprecated functions * * */
  // Point-plotting functions
  void draw_half_ellipse (triple, triple);
  // Stanza-creating whole ellipse functions
  void draw_ellipse (triple, triple); 
  void ellipse (triple, triple);
  // Stanza-creating standard half-ellipse functions
  void draw_half_ellipse(triple, triple, double);
  void ellipse_left (triple, triple);
  void ellipse_right (triple, triple);
  void ellipse_top (triple, triple);
  void ellipse_bottom (triple, triple);
  // Right half ellipse rotated through <double> revolutions
  //  void rotate_half_ellipse(triple, triple, double);
  //  void ellipse_half (triple, triple, double);

  // Circular arcs parallel to (x,y)-plane
  void draw_arc (triple, double, double, double);

  void arc (triple, double, double, double);

  void draw_arc_arrow (triple center, double r, double start, double finish, 
		       double scale=1);
  void arc_arrow (triple center, double r, double start, double finish,
		  double scale=1);
  // End of deprecated functions

  /* * * 3-D ellipse functions * * */
  void draw_ellipse_arc(triple center, triple axis1, triple axis2, 
			double t_min, double t_max, int num_pts);
  void draw_ellipse(triple center, triple axis1, triple axis2, 
		    int num_pts=EPIX_NUM_PTS);
  void ellipse_arc(triple center, triple axis1, triple axis2, 
		   double t_min, double t_max, int num_pts);
  void ellipse(triple center, triple axis1, triple axis2, 
	       int num_pts=EPIX_NUM_PTS);

  // Quadratic and cubic splines
  void draw_spline(triple p1, triple p2, triple p3,
		   int num_pts=EPIX_NUM_PTS);
  void draw_spline(triple p1, triple p2, triple p3, triple p4,
		   int num_pts=EPIX_NUM_PTS);

  void spline (triple p1, triple p2, triple p3,
	       int num_pts=EPIX_NUM_PTS);
  void spline (triple p1, triple p2, triple p3, triple p4,
	       int num_pts=EPIX_NUM_PTS);

  /*
  // Knot diagram primitives
  void draw_under_strand(triple p3, triple p6, triple p5, triple p2);
  void draw_p_twist(triple p1, triple p4);
  void draw_n_twist(triple p1, triple p4);

  void p_twist (triple, triple);
  void n_twist (triple, triple);
  void twists (triple, triple, int);
  */

} /* end of namespace */

/*
 * plots.h -- function plotting
 *
 * Andrew D. Hwang            <ahwang@mathcs.holycross.edu>
 *
 * Version 0.8.10rc10
 * Last Change: May 26, 2003
 *
 */

/* 
 * Copyright (C) 2001, 2002, 2003
 * Andrew D. Hwang <ahwang@mathcs.holycross.edu>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */



namespace ePiX {
  // utility functions
  inline void plot_pt(double f(double), double t) 
    { print(P(t, f(t), 0)); }
  inline void plot_pt(double f1(double), double f2(double), double t)
    { print(P(f1(t), f2(t), 0)); }
  inline void plot_pt(double f1(double), double f2(double), 
			double f3(double), double t) 
    { print(P(f1(t), f2(t), f3(t))); }

  inline void plot_pt(pair F(double), double t) 
    { print(P(F(t).x1, F(t).x2, 0)); }
  inline void plot_pt(triple F(double), double t) 
    { print(F(t)); }

  inline void polar_plot_pt(double f(double), double t) 
    { print(polar(f(t),t)); }

  // Adaptive plotting
  void draw_adapt_plot (double f1(double), double f2(double), 
			double, double, int);
  void adplot     (double f(double), double, double, int);
  void adplot     (double f1(double), double f2(double), double, double, int);

  void draw_adapt_plot (triple f(double), double, double, int);
  void adplot     (triple f(double), double, double, int);

  // Ordinary graph plot
  void draw_plot (double f(double), double, double, int);
  void plot      (double f(double), double, double, int);

  // Parametric plane curve
  void draw_plot (double f1(double), double f2(double), double, double, int);
  void plot      (double f1(double), double f2(double), double, double, int);

  void draw_plot (triple f(double), double, double, int);
  void plot      (triple f(double), double, double, int);

  // Parametric space curve
  void draw_plot (double f1(double), double f2(double), double f3(double),
		  double, double, int);

  void plot      (double f1(double), double f2(double), double f3(double),
		  double, double, int);

  void draw_plot (triple f(double), double, double, int);
  void plot      (triple f(double), double, double, int);

  /*
  // Projection of space curve to plane through the origin
  void draw_shadowplot (double f1(double), double f2(double), 
			double f3(double), triple, double, double, int);
  void shadow          (double f1(double), double f2(double), 
			double f3(double), triple, double, double, int);

  void draw_shadowplot (triple f(double), triple, double, double, int);
  void shadow          (triple f(double), triple, double, double, int);
  */

  // Polar graph
  void draw_polarplot (double r(double), double, double, int);
  void polarplot      (double r(double), double, double, int);

  // Plotting families of curves -- sliceplot1 holds first variable constant
  void draw_sliceplot1(double f1(double, double), double f2(double, double), 
		       double, double, double, int);
  void draw_sliceplot2(double f1(double, double), double f2(double, double), 
		       double, double, double, int);

  void sliceplot1 (double f1(double, double), double f2(double, double), 
		   double, double, double, int);
  void sliceplot2 (double f1(double, double), double f2(double, double), 
		   double, double, double, int);

  // triple-valued plot argument
  void draw_sliceplot1(triple f(double, double), double, double, double, int);
  void draw_sliceplot2(triple f(double, double), double, double, double, int);
  void sliceplot1 (triple f(double, double), double, double, double, int);
  void sliceplot2 (triple f(double, double), double, double, double, int);

  // multiplot = several sliceplots
  void draw_multiplot1 (double f1(double, double), double f2(double, double), 
			triple, triple, mesh, int);
  void draw_multiplot2 (double f1(double, double), double f2(double, double), 
			triple, triple, mesh, int);

  void multiplot1 (double f1(double, double), double f2(double, double), 
		   triple, triple, mesh, int);
  void multiplot2 (double f1(double, double), double f2(double, double), 
		   triple, triple, mesh, int);
  // triple-valued plot argument
  void draw_multiplot1 (triple f(double, double), triple, triple, mesh, int);
  void draw_multiplot2 (triple f(double, double), triple, triple, mesh, int);
  void multiplot1 (triple f1(double, double), triple, triple, mesh, int);
  void multiplot2 (triple f1(double, double), triple, triple, mesh, int);

  // "Cropped" plotting -- truncated to bounding_box
  void draw_cropplot (double f1(double), double f2(double), double f3(double),
		      double, double, int);
  void draw_cropplot (triple f(double), double, double, int);
  void draw_cropplot (triple f(double), double, double, int);

  // Ordinary plots
  void cropplot (double f(double), double, double, int);
  // Parametric plane curves
  void cropplot (double f1(double), double f2(double), double, double, int);
  // Parametric space curves
  void cropplot (double f1(double), double f2(double), double f3(double),
		 double t_min, double t_max, int num_pts);
  void cropplot (triple f(double), double t_min, double t_max, int num_pts);

  // For backward compatibility
  inline void clipplot (double f(double), double t_min, double t_max, 
			int num_pts) { cropplot(f, t_min, t_max, num_pts); }

  inline void clipplot (double f1(double), double f2(double), 
			double t_min, double t_max, int num_pts)
    { cropplot(f1, f2, t_min, t_max, num_pts); }

  inline void clipplot (double f1(double),double f2(double),double f3(double),
			double t_min, double t_max, int num_pts)
    { cropplot(f1, f2, f3, t_min, t_max, num_pts); }

  inline void clipplot (triple f(double),double t_min,double t_max,int num_pts)
    { cropplot(f, t_min, t_max, num_pts); }

  // Data plotting from file
  void data_plot (const char *, epix_mark_type);

  // Derivatives and integrals
  void draw_deriv     (double f(double), double, double, int);
  void plot_deriv     (double f(double t), double, double, int);

  void tan_line(double f1(double t), double f2(double t), double t0);
  void tan_line(double f(double t), double t0);
  void tan_line(triple f(double t), double t0);

  void envelope(double f1(double), double f2(double), double, double, int);
  void envelope(double f(double), double, double, int);
  void envelope(triple f(double), double, double, int);

  void tan_field     (double f1(double), double f2(double), 
		      double, double, double);
  //  void boldtan_field (double f1(double), double f2(double), 
  //		      double, double, double);
  void tan_field     (triple f(double), double, double, double);
  //  void boldtan_field (triple f(double), double, double, double);

  // lower limit is endpoint of plot interval
  double integrate  (double f(double), double a, double b);
  void draw_int     (double f(double), double, double, int);
  void plot_int     (double f(double), double, double, int);

  // separate lower limit
  void draw_int     (double f(double), double, double, double, int);
  void plot_int     (double f(double), double, double, double, int);

  // Slope, dart, and vector fields
  void draw_slope_field (double f1(double, double), double f2(double, double),
			 triple, triple, int, int);
  void slope_field      (double f1(double, double), double f2(double, double),
			 triple, triple, int, int);
  void boldslope_field  (double f1(double, double), double f2(double, double),
			 triple, triple, int, int);

  void draw_dart_field  (double f1(double, double), double f2(double, double),
			 triple, triple, int, int);
  void dart_field       (double f1(double, double), double f2(double, double),
			 triple, triple, int, int);
  void bolddart_field   (double f1(double, double), double f2(double, double),
			 triple, triple, int, int);

  void draw_vector_field (double f1(double, double), double f2(double, double),
			  triple, triple, int, int);
  void vector_field      (double f1(double, double), double f2(double, double),
			  triple, triple, int, int);
  void boldvector_field  (double f1(double, double), double f2(double, double),
			  triple, triple, int, int);

  // triple-valued vector field
  void draw_slope_field  (triple F(double, double), triple, triple, int, int);
  void slope_field       (triple F(double, double), triple, triple, int, int);
  void boldslope_field   (triple F(double, double), triple, triple, int, int);

  void draw_dart_field   (triple F(double, double), triple, triple, int, int);
  void dart_field        (triple F(double, double), triple, triple, int, int);
  void bolddart_field    (triple F(double, double), triple, triple, int, int);

  void draw_vector_field (triple F(double, double), triple, triple, int, int);
  void vector_field      (triple F(double, double), triple, triple, int, int);
  void boldvector_field  (triple F(double, double), triple, triple, int, int);

  // Solutions of ODE systems
  void draw_ode_plot (double f1(double, double), double f2(double, double),
		      triple, double, int);
  void ode_plot      (double f1(double, double), double f2(double, double),
		      triple, double, int);

  // triple-valued vector field
  void draw_ode_plot (triple F(double, double), triple, double, int);
  void ode_plot      (triple F(double, double), triple, double, int);

  // Image of a point set under a planar flow
  void draw_flow_plot(double f1(double, double), double f2(double, double),
		      triple f(double), triple color(double), double, int, int);
  // With and without color function
  void flow_plot(double f1(double, double), double f2(double, double),
		 triple f(double), triple color(double), double, int, int);
  void flow_plot(double f1(double, double), double f2(double, double),
		 triple f(double), double, int, int);

  // triple-valued vector field
  void draw_flow_plot(triple F(double, double), triple f(double), 
		      triple color(double), double, int, int);
  void flow_plot(triple F(double, double), triple f(double), 
		 triple color(double), double, int, int);
  void flow_plot(triple F(double, double), triple f(double), double, int, int);

  // Jay Belanger's shaded plot routines -- December 1, 2002

  void blackplot(double f(double), double, double, int);
  void blackplot(double f1(double), double f2(double), double, double, int);

  void whiteplot(double f(double), double, double, int);
  void whiteplot(double f1(double), double f2(double), double, double, int);

  void shadeplot(double f(double), double, double, int);
  void shadeplot(double f1(double), double f2(double), double, double, int);

} /* end of namespace */

/*
 * geometry.h 
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 0.8.9rc1
 * Last Change: January 30, 2003
 */

/* 
 * Copyright (C) 2001, 2002, 2003
 * Andrew D. Hwang <ahwang@mathcs.holycross.edu>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */




namespace ePiX {

  // Radial projection from origin to unit sphere
  void plot_R      (double f1(double), double f2(double), double f3(double), 
		    double t_min, double t_max, int num_pts);
  void frontplot_R (double f1(double), double f2(double), double f3(double), 
		    double t_min, double t_max, int num_pts);
  void backplot_R  (double f1(double), double f2(double), double f3(double), 
		    double t_min, double t_max, int num_pts);

  // triple-valued plot argument
  void plot_R      (triple phi(double), 
		    double t_min, double t_max, int num_pts);
  void frontplot_R (triple phi(double), 
		    double t_min, double t_max, int num_pts);
  void backplot_R  (triple phi(double), 
		    double t_min, double t_max, int num_pts);

  // Stereographic projection from the north pole...
  void plot_N      (double f1(double), double f2(double),
		    double t_min, double t_max, int num_pts);
  void frontplot_N (double f1(double), double f2(double),
		    double t_min, double t_max, int num_pts);
  void backplot_N  (double f1(double), double f2(double),
		    double t_min, double t_max, int num_pts);

  // and the south
  void plot_S      (double f1(double), double f2(double),
		    double t_min, double t_max, int num_pts);
  void frontplot_S (double f1(double), double f2(double),
		    double t_min, double t_max, int num_pts);
  void backplot_S  (double f1(double), double f2(double),
		    double t_min, double t_max, int num_pts);

  // Hyperbolic lines
  void draw_hyperbolic_line (pair, pair);
  void hyperbolic_line (pair, pair);

  void draw_disk_line (pair, pair);
  void disk_line (pair, pair);

} /* end of namespace */

/*
 * arcana.h
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 0.8.10rc5
 * Last Change: April 14, 2003
 */

/* 
 * Copyright (C) 2001, 2002, 2003
 * Andrew D. Hwang <ahwang@mathcs.holycross.edu>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */




namespace ePiX {

  // Implementation of the momentum construction
  void draw_profile (double f(double), double, double, int);
  void plot_profile (double f(double), double, double, int);
  void plot_profile (double f(double), double, double, double, int);

  // Plane linear transformations
  pair 
    transf (pair, pair, pair);

  pair 
    reflect (double, pair);

  // Image of the "standard F" under a linear transformation
  void 
    std_F (pair, pair);

  /*
   * fractal generation
   *
   * The basic "level-1" recursion unit is a piecewise-linear path whose 
   * segments are parallel to spokes on a wheel, labelled modulo <spokes>.
   * Recursively up to <depth>, each segment is replaced by a copy of the
   * recursion unit.
   *
   * Sample data for _/\_ standard Koch snowflake:
   * const int pre_seed[] = {6, 4, 0, 1, -1, 0};
   * pre_seed[0] = spokes, pre_seed[1] = seed_length;
   */

  void 
    fractal (triple p, triple q, int depth, const int *pre_seed);

} /* end of namespace */

/*
 * meshplots.h
 *
 * This file is part of ePiX, a preprocessor for creating high-quality 
 * line figures in LaTeX 
 *
 * Version 0.8.10rc6
 * Last Change: April 16, 2003
 */

/* 
 * Copyright (C) 2001, 2002, 2003
 * Andrew D. Hwang <ahwang@mathcs.holycross.edu>
 * Department of Mathematics and Computer Science
 * College of the Holy Cross
 * Worcester, MA, 01610-2395, USA
 */

/*
 * ePiX is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * ePiX is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
 * License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ePiX; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */




namespace ePiX {

  // meshplots.cc

  void draw_plot(double f1(double, double),
		     double f2(double, double),
		     double f3(double, double), 
		     triple p1, triple p2, mesh N, mesh num_pts);

  // Parametric surfaces
  void plot(double f1(double, double), 
		double f2(double, double), 
		double f3(double, double), 
		triple p1, triple p2, mesh N, mesh num_pts);

  // Graphs of functions of two real variables
  void plot(double f(double, double), 
		triple p1, triple p2, mesh N, mesh num_pts);
  
  // triple-valued plot argument
  void draw_plot(triple Phi(double, double), 
		     triple p1, triple p2, mesh N, mesh num_pts);
  void plot     (triple Phi(double, double), 
		     triple p1, triple p2, mesh N, mesh num_pts);

  // clipped wire mesh plots
  void clipplot(double f1(double, double), 
		double f2(double, double), 
		double f3(double, double), 
		triple p1, triple p2, mesh N, mesh num_pts);
  void clipplot(triple Phi(double, double), 
		triple p1, triple p2, mesh N, mesh num_pts);

  /* * * backward compatibility * * */
  inline void draw_wiremesh(double f1(double, double),
			    double f2(double, double),
			    double f3(double, double), 
			    triple p1, triple p2, mesh N, mesh num_pts)
    { draw_plot(f1,f2,f3, p1,p2, N, num_pts); }

  // Parametric surfaces
  inline void wiremesh(double f1(double, double), 
		       double f2(double, double), 
		       double f3(double, double), 
		       triple p1, triple p2, mesh N, mesh num_pts)
    { plot(f1,f2,f3, p1,p2, N, num_pts); }

  // Graphs of functions of two real variables
  inline void wiremesh(double f(double, double), 
		       triple p1, triple p2, mesh N, mesh num_pts)
    { plot(f, p1,p2, N, num_pts); }
  
  // triple-valued plot argument
  inline void draw_wiremesh(triple Phi(double, double), 
			    triple p1, triple p2, mesh N, mesh num_pts)
    { draw_plot(Phi, p1,p2, N, num_pts); }

  inline void wiremesh     (triple Phi(double, double), 
			    triple p1, triple p2, mesh N, mesh num_pts)
    { plot(Phi, p1, p2, N, num_pts); }

  // clipped wire mesh plots
  inline void clipmesh(double f1(double, double), 
		       double f2(double, double), 
		       double f3(double, double), 
		       triple p1, triple p2, mesh N, mesh num_pts)
    { clipplot(f1,f2,f3, p1,p2, N,num_pts); }

  inline void clipmesh(triple Phi(double, double), 
		       triple p1, triple p2, mesh N, mesh num_pts)
    { clipplot(Phi, p1,p2, N,num_pts); }

} /* end of namespace */

